"""
Silique (https://www.silique.fr)
Copyright (C) 2024-2025

This program is free software: you can redistribute it and/or modify it
under the terms of the GNU Lesser General Public License as published by the
Free Software Foundation, either version 3 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
details.

You should have received a copy of the GNU Lesser General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from pathlib import Path
from .i18n import _


def get_rougail_config(
    *,
    backward_compatibility=True,
) -> dict:
    options = f"""
json:
  description: {_('configuration rougail-json')}
  disabled:
    type: jinja
    jinja: |
      {{% if step.output is propertyerror or step.output != 'json' %}}
      disabled
      {{% endif %}}

  read_write:
    description: {_('display variables available in read_write mode')}
    negative_description: {_('display variables available in read_only mode')}
    default: false

  mandatory:
    description: {_('test mandatories variable before display in json')}
    negative_description: {_('do not test mandatories variable before display in json')}
    default: true

  get:
    description: {_('get value for a variable or a family')}
    mandatory: false
"""
    return {
        "name": "json",
        "process": "output",
        "options": options,
        "level": 50,
    }


__all__ = ("get_rougail_config",)
